#include "font.h"
#include "main.h"
#include "utils.h"


Font::Font(const std::string &name, int ptsize)
{
    int size;

    data = resources.getRef(name, &size);
    if (! data)
        throw Exception(name + " not found");
    SDL_RWops *op = SDL_RWFromMem(data, size);
    font = TTF_OpenFontRW(op, 1, ptsize);

    if (! font)
        throw Exception("Error loading font " + name);
}


Font::~Font()
{
    TTF_CloseFont(font);
    resources.delRef(data);
}


void Font::draw(SDL_Surface *s, int x, int y, int r, int g, int b, bool shadow, const std::string &text)
{
    if (text.length() < 1)
        return;
    
    if (shadow) {
        SDL_Color color = { 0, 0, 0, 0 };
        SDL_Surface *surface = TTF_RenderText_Blended(font, text.c_str(), color);
        SDL_Rect src = { 0, 0, surface->w, surface->h };
        SDL_Rect dst = { x+1, y+1, surface->w, surface->h };
        SDL_BlitSurface(surface, &src, s, &dst);
        SDL_FreeSurface(surface);
    }
    SDL_Color color = { r, g, b, 0 };
    SDL_Surface *surface = TTF_RenderText_Blended(font, text.c_str(), color);
    SDL_Rect src = { 0, 0, surface->w, surface->h };
    SDL_Rect dst = { x, y, surface->w, surface->h };
    SDL_BlitSurface(surface, &src, s, &dst);
    SDL_FreeSurface(surface);
}

void Font::draw(int x, int y, int r, int g, int b, bool shadow, const std::string &text)
{
    draw(screen.getSurface(), x, y, r,g,b, shadow, text);
}

int Font::getWidth(const std::string &text)
{
    int w, h;
    TTF_SizeText(font, text.c_str(), &w, &h);
    return w;
}


int Font::getHeight(const std::string &text)
{
    int w, h;
    TTF_SizeText(font, text.c_str(), &w, &h);
    return h;
}

void Font::getSize(const std::string &text, int &width, int &height)
{
    TTF_SizeText(font, text.c_str(), &width, &height);
}

