#include <stdlib.h>
#include <iostream>
#include <SDL_ttf.h>
#include "main.h"
#include "utils.h"
#include "storage.h"


Screen screen;
Resources resources;
Random rndGen;


static SDL_Cursor *cursor = NULL;


static void initScreen()
{
    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO) < 0)
        throw Exception(std::string("Error initializing SDL: ") + SDL_GetError());
    atexit(SDL_Quit);
    if (TTF_Init())
        throw Exception("Error initializing font engine");
    screen.setMode(VideoMode(800, 600, 24, getStorage()->get("fullscreen", 1) != 0));

    //SDL_ShowCursor(0);
    cursor = SDL_GetCursor();
    Uint8 t = 0;
    SDL_Cursor *c = SDL_CreateCursor(&t, &t, 8, 1, 0, 0);
    SDL_SetCursor(c);
    
    SDL_Surface *mouse = loadImage("cursor.bmp");
    SDL_SetColorKey(mouse, SDL_SRCCOLORKEY, SDL_MapRGB(mouse->format, 0, 0, 0));
    screen.setMouseImage(mouse);
    SDL_FreeSurface(mouse);
    SDL_WM_SetCaption("Einstein", NULL);
}


static void loadResources()
{
    try {
#ifdef WIN32
        std::string fileName = getStorage()->get("path", "") + "\\einstein.res";
#else
        std::string fileName = PREFIX "/share/einstein/einstein.res";
#endif
        resources.open(fileName);
    } catch (...) {
        resources.open("einstein.res");
    }
}

int main(int argc, char *argv[])
{
#ifndef WIN32
    ensureDirExists(getenv("HOME") + std::string("/.einstein"));
#endif
    
 /*   std::cout << "Locale: '" << locale.getLanguage() << "' '" << 
        locale.getCountry() << "' '" << locale.getEncoding() << "'" << std::endl;*/
    
    try {
        loadResources();
        initScreen();
        menu();
        getStorage()->flush();
    } catch (Exception &e) {
        std::cerr << "ERROR: " << e.getMessage() << std::endl;
    } catch (...) {
        std::cerr << "ERROR: Unknown exception" << std::endl;
    }
    if (cursor)
        SDL_SetCursor(cursor);
    
    return 0;
}

