#include <vector>
#include "main.h"
#include "utils.h"
#include "widgets.h"
#include "topscores.h"
#include "opensave.h"
#include "game.h"
#include "descr.h"
#include "options.h"



class MenuBackground: public Widget
{
    public:
        virtual void draw();
};


void MenuBackground::draw()
{
  /*  drawWallpaper("marble1.bmp");
    SDL_Surface *title = loadImage("menutitle.bmp");
    screen.draw(140, 28, title);
    SDL_FreeSurface(title);*/
    
    SDL_Surface *title = loadImage("nova.bmp");
    screen.draw(0, 0, title);
    SDL_FreeSurface(title);
    screen.addRegionToUpdate(0, 0, screen.getWidth(), screen.getHeight());
}



class NewGameCommand: public Command
{
    private:
        Area *area;
    
    public:
        NewGameCommand(Area *a) { area = a; };

        virtual void doAction() {
            Game game;
            game.run();
            area->updateMouse();
            area->draw();
        };
};


class LoadGameCommand: public Command
{
    private:
        Area *area;
    
    public:
        LoadGameCommand(Area *a) { area = a; };

        virtual void doAction() {
            Game *game = loadGame(area);
            if (game) {
                game->run();
                delete game;
            }
            area->updateMouse();
            area->draw();
        };
};



class TopScoresCommand: public Command
{
    private:
        Area *area;
    
    public:
        TopScoresCommand(Area *a) { area = a; };

        virtual void doAction() {
            TopScores scores;
            showScoresWindow(area, &scores);
            area->updateMouse();
            area->draw();
        };
};


class RulesCommand: public Command
{
    private:
        Area *area;
    
    public:
        RulesCommand(Area *a) { area = a; };

        virtual void doAction() {
            showDescription(area);
            area->updateMouse();
            area->draw();
        };
};


class OptionsCommand: public Command
{
    private:
        Area *area;
    
    public:
        OptionsCommand(Area *a) { area = a; };

        virtual void doAction() {
            showOptionsWindow(area);
            area->updateMouse();
            area->draw();
        };
};


class AboutCommand: public Command
{
    private:
        Area *parentArea;
    
    public:
        AboutCommand(Area *a) { parentArea = a; };

        virtual void doAction() {
            Area area;
            Font titleFont("cosm.ttf", 26);
            Font font("luxisb.ttf", 14);

#define LABEL(pos, c, text) area.add(new Label(&font, 220, pos, 360, 20, \
            Label::ALIGN_CENTER, Label::ALIGN_MIDDLE, 255,255,c, text));
            area.add(parentArea);
            area.add(new Window(220, 160, 360, 280, "blue.bmp"));
            area.add(new Label(&titleFont, 250, 165, 300, 40, Label::ALIGN_CENTER,
                        Label::ALIGN_MIDDLE, 255,255,0, "ABOUT"));
            LABEL(240, 255, "Einstein Puzzle")
            LABEL(260, 255, "version 1.0")
            LABEL(280, 255, "Copyright (c) 2003-2004 Alexander Babichev")
            LABEL(330, 0, "http://www.babichev.info")
#undef LABEL
            ExitCommand exitCmd(area);
            area.add(new Button(360, 400, 80, 25, &font, 255,255,0, "blue.bmp", 
                        "OK", &exitCmd));
            area.add(new KeyAccel(SDLK_ESCAPE, &exitCmd));
            area.add(new KeyAccel(SDLK_RETURN, &exitCmd));
            area.run();

            parentArea->updateMouse();
            parentArea->draw();
        };
};


static Button* menuButton(int y, Font *font, const std::string &text, 
        Command *cmd=NULL)
{
    return new Button(550, y, 220, 30, font, 0,240,240, 30,255,255, text, cmd);
}


void menu()
{
    Area area;
    Font font("luxisb.ttf", 20);

    area.add(new MenuBackground());
    area.draw();

    NewGameCommand newGameCmd(&area);
    area.add(menuButton(355, &font, "Start New Game", &newGameCmd));
    LoadGameCommand loadGameCmd(&area);
    area.add(menuButton(385, &font, "Load Game", &loadGameCmd));
    TopScoresCommand topScoresCmd(&area);
    area.add(menuButton(415, &font, "Hall Of Fame", &topScoresCmd));
    /*RulesCommand rulesCmd(&area);
    area.add(menuButton(430, &font, "Rules", &rulesCmd));*/
    OptionsCommand optionsCmd(&area);
    area.add(menuButton(445, &font, "Options", &optionsCmd));
    AboutCommand aboutCmd(&area);
    area.add(menuButton(475, &font, "About", &aboutCmd));
    ExitCommand exitMenuCmd(area);
    area.add(menuButton(505, &font, "Exit", &exitMenuCmd));
    area.add(new KeyAccel(SDLK_ESCAPE, &exitMenuCmd));

    area.run();
}

