#include "options.h"
#include "storage.h"
#include "main.h"


class OptionsChangedCommand: public Command
{
    private:
        bool &fullscreen;
        Area *area;
    
    public:
        OptionsChangedCommand(Area *a, bool &fs): fullscreen(fs) {
            area = a;
        };

        virtual void doAction() {
            bool oldFullscreen = (getStorage()->get("fullscreen", 1) != 0);
            if (fullscreen != oldFullscreen) {
                getStorage()->set("fullscreen", fullscreen);
                getStorage()->flush();
                screen.setMode(VideoMode(800, 600, 24, fullscreen));
            }
            area->finishEventLoop();
        };
};


void showOptionsWindow(Area *parentArea)
{
    Font titleFont("cosm.ttf", 26);
    Font font("luxisb.ttf", 14);

    bool fullscreen = (getStorage()->get("fullscreen", 1) != 0);
    
    Area area;

    area.add(parentArea);
    area.add(new Window(250, 200, 300, 200, "blue.bmp"));
    area.add(new Label(&titleFont, 250, 205, 300, 40, Label::ALIGN_CENTER,
                Label::ALIGN_MIDDLE, 255,255,0, "OPTIONS"));
    area.add(new Checkbox(265, 290, 20, 20, &font, 255,255,255, "blue.bmp", 
                fullscreen));
    area.add(new Label(&font, 300, 290, 300, 20, Label::ALIGN_LEFT,
                Label::ALIGN_MIDDLE, 255,255,255, "Run in fullscreen mode"));
    ExitCommand exitCmd(area);
    OptionsChangedCommand okCmd(&area, fullscreen);
    area.add(new Button(315, 360, 85, 25, &font, 255,255,0, "blue.bmp", 
                "OK", &okCmd));
    area.add(new Button(405, 360, 85, 25, &font, 255,255,0, "blue.bmp", 
                "Cancel", &exitCmd));
    area.add(new KeyAccel(SDLK_ESCAPE, &exitCmd));
    area.add(new KeyAccel(SDLK_RETURN, &okCmd));
    area.run();
}

