#ifdef WIN32                    // Win32 only
#include "regstorage.h"


RegistryStorage::RegistryStorage()
{
    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, 
                "SOFTWARE\\babichev\\einstein\\1.0",
                0, KEY_READ, &globalKey))
        globalKey = NULL;
    if (RegOpenKeyEx(HKEY_CURRENT_USER, 
                "SOFTWARE\\babichev\\einstein\\1.0",
                0, KEY_READ | KEY_WRITE, &userKey))
    {
        if (RegCreateKeyEx(HKEY_CURRENT_USER, 
                    "SOFTWARE\\babichev\\einstein\\1.0", 0, NULL, 
                    REG_OPTION_NON_VOLATILE, KEY_READ | KEY_WRITE,
                    NULL, &userKey, NULL))
            userKey = NULL;
    }
}

RegistryStorage::~RegistryStorage()
{
    if (globalKey)
        RegCloseKey(globalKey);
    if (userKey)
        RegCloseKey(userKey);
}

int RegistryStorage::get(const std::string &name, int dflt)
{
    if (globalKey) {
        DWORD data;
        DWORD size = sizeof(data);
        DWORD type;
        if (! RegQueryValueEx(globalKey, name.c_str(), NULL, &type,
                    (BYTE*)&data, &size))
            if (type == REG_DWORD)
                return data;
    }
    
    if (userKey) {
        DWORD data;
        DWORD size = sizeof(data);
        DWORD type;
        if (! RegQueryValueEx(userKey, name.c_str(), NULL, &type,
                    (BYTE*)&data, &size))
            if (type == REG_DWORD)
                return data;
    }
    
    return dflt;
}

std::string RegistryStorage::get(const std::string &name, const std::string &dflt)
{
    if (globalKey) {
        DWORD size = 0;
        DWORD type;
        if (! RegQueryValueEx(globalKey, name.c_str(), NULL, &type,
                    NULL, &size)) 
        {
            if ((type == REG_SZ) && (size > 0)) {
                char *data = new char[size];
                if (! RegQueryValueEx(globalKey, name.c_str(), NULL, &type,
                            (BYTE*)data, &size)) 
                {
                    std::string s(data);
                    delete[] data;
                    return s;
                }
                delete[] data;
            } else
                return "";
        }
    }
    
    if (userKey) {
        DWORD size = 0;
        DWORD type;
        if (! RegQueryValueEx(userKey, name.c_str(), NULL, &type,
                    NULL, &size)) 
        {
            if ((type == REG_SZ) && (size > 0)) {
                char *data = new char[size];
                if (! RegQueryValueEx(userKey, name.c_str(), NULL, &type,
                            (BYTE*)data, &size)) 
                {
                    std::string s(data);
                    delete[] data;
                    return s;
                }
                delete[] data;
            } else
                return "";
        }
    }
    
    return dflt;
}

void RegistryStorage::set(const std::string &name, int value)
{
    if (userKey) {
        DWORD data = value;
        RegSetValueEx(userKey, name.c_str(), 0, REG_DWORD, 
                (BYTE*)&data, sizeof(data));
    }
}

void RegistryStorage::set(const std::string &name, const std::string &value)
{
    if (userKey)
        RegSetValueEx(userKey, name.c_str(), 0, REG_SZ, 
                (BYTE*)value.c_str(), value.length() + 1);
}


#endif

