#ifndef __RESOURCES_H__
#define __RESOURCES_H__

#include <string>
#include <set>
#include <map>
#include <fstream>
#include "exceptions.h"


class Resources
{
    private:
        class Resource
        {
            public:
                std::string name;
                long size;
                long offset;
                long packedSize;
                int comprLevel;
                char* buf;
                int references;

            public:
                Resource(const std::string& name, long size, long offset, long packedSize, int cmprLevel);
                virtual ~Resource();
        };
        
        class ResLessThen
        {
            public:
                int operator() (const Resource* i1, const Resource* i2) const {
                    return i1->name < i2->name;
                }
        };
    
    private:
        std::ifstream stream;
    
    private:
        std::map<std::string, Resource*, std::less<std::string> > directory;
        std::set<Resource*, Resources::ResLessThen> loaded;
    
    public:
        Resources();
        ~Resources();

    public:
        void open(const std::string& fileName);
        void* getRef(const std::string& name, int *size=NULL) throw(Exception);
        void delRef(void *buf);

    private:
        Resource* findResource(const std::string &fileName);
};

#endif

