#include <fstream>
#include "tablestorage.h"


TableStorage::TableStorage()
{
    conf = table_read(getFileName().c_str());
    if (! conf)
        conf = table_create();
}

TableStorage::~TableStorage()
{
    flush();
    table_free(conf);
}

std::string TableStorage::getFileName()
{
#ifndef WIN32
    return std::string(getenv("HOME")) + "/.einstein/einsteinrc";
#else
    return "einstein.cfg";
#endif
}

int TableStorage::get(const std::string &name, int dflt)
{
    return table_get_int(conf, (char*)name.c_str(), dflt, NULL);
}

std::string TableStorage::get(const std::string &name, 
            const std::string &dflt)
{
    return std::string(table_get_strs(conf, (char*)name.c_str(),
                (char*)dflt.c_str(), NULL));
}

void TableStorage::set(const std::string &name, int value)
{
    table_set_int(conf, name.c_str(), value);
}

void TableStorage::set(const std::string &name, const std::string &value)
{
    table_set_str(conf, name.c_str(), value.c_str());
}

void TableStorage::flush()
{
    char *s = table_to_str(conf, 0, 1, 0);
    if (s) {
        std::ofstream of(getFileName().c_str());
        of << s;
        of.close();
        free(s);
    }
}

