#include "topscores.h"
#include "storage.h"
#include "utils.h"
#include "font.h"


TopScores::TopScores()
{
    Storage *storage = getStorage();
    
    for (int i = 0; i < MAX_SCORES; i++) {
        int score = storage->get("top_score_" + numToStr(i), -1);
        if (score < 0)
            break;
        std::string name = storage->get("top_name_" + numToStr(i), "");
        add(name, score);
    }
    
    modifed = false;
}


TopScores::~TopScores()
{
    save();
}

int TopScores::add(const std::string &name, int score)
{
    if (score >= getMaxScore() || (scores.size() < 1)) {
        if (! isFull()) {
            Entry e = { name, score };
            scores.push_back(e);
            modifed = true;
            return scores.size() - 1;
        }
        return -1;
    }
    
    int pos = 0;
    for (ScoresList::iterator i = scores.begin(); i != scores.end(); i++) {
        Entry &e = *i;
        if (e.score > score) {
            Entry ne = { name, score };
            scores.insert(i, ne);
            modifed = true;
            break;
        }
        pos++;
    }

    while (scores.size() > MAX_SCORES) {
        modifed = true;
        scores.erase(--scores.end());
    }

    return modifed ? pos : -1;
}

void TopScores::save()
{
    if (! modifed)
        return;

    Storage *storage = getStorage();
    int no = 0;
    
    for (ScoresList::iterator i = scores.begin(); i != scores.end(); i++) {
        Entry &e = *i;
        storage->set("top_name_" + numToStr(no), e.name);
        storage->set("top_score_" + numToStr(no), e.score);
        no++;
    }
    
    storage->flush();
    modifed = false;
}

TopScores::ScoresList& TopScores::getScores()
{
    return scores;
}

int TopScores::getMaxScore()
{
    if (scores.size() < 1)
        return -1;
    ScoresList::iterator i = scores.end();
    i--;
    return (*i).score;
}


class ScoresWindow: public Window
{
    public:
        ScoresWindow(int x, int y, TopScores *scores, int highlight);
};


ScoresWindow::ScoresWindow(int x, int y, TopScores *scores, int highlight): 
                Window(x, y, 320, 350, "blue.bmp")
{
    Font titleFont("cosm.ttf", 26);
    Font entryFont("luxisb.ttf", 14);
    Font timeFont("luximb.ttf", 14);
    
    titleFont.draw(background, 53, 15, 255,255,0, true, "HALL OF FAME");

    TopScores::ScoresList &list = scores->getScores();
    int no = 1;
    int pos = 70;
    for (TopScores::ScoresList::iterator i = list.begin(); 
            i != list.end(); i++) 
    {
        TopScores::Entry &e = *i;
        std::string s(numToStr(no) + ".");
        int w = entryFont.getWidth(s);
        int c = ((no - 1) == highlight) ? 0 : 255;
        entryFont.draw(background, 30 - w, pos, 255,255,c, true, s);
        SDL_Rect rect = { 40, pos-20, 180, 40 };
        SDL_SetClipRect(background, &rect);
        entryFont.draw(background, 40, pos, 255,255,c, true, e.name);
        SDL_SetClipRect(background, NULL);
        s = secToStr(e.score);
        w = timeFont.getWidth(s);
        timeFont.draw(background, 305-w, pos, 255,255,c, true, s);
        pos += 20;
        no++;
    }
}


void showScoresWindow(Area *parentArea, TopScores *scores, int highlight)
{
    Area area;

    area.add(parentArea);
    area.add(new ScoresWindow(240, 125, scores, highlight));
    ExitCommand exitCmd(area);
    area.add(new Button(353, 425, "flatokbtn.bmp", &exitCmd, false));
    area.add(new KeyAccel(SDLK_ESCAPE, &exitCmd));
    area.run();
}


std::string enterNameDialog(Area *parentArea)
{
    Area area;
    
    Font font("luxisb.ttf", 16);
    area.add(parentArea);
    area.add(new Window(170, 280, 460, 100, "blue.bmp"));
    Storage *storage = getStorage();
    std::string name = storage->get("lastName", "Anonymous");
    area.add(new Label(&font, 180, 300, 255,255,0, "Enter your name:"));
    area.add(new InputField(340, 300, 280, 26, "blue.bmp", name, 20,  
                255,255,0,  &font));
    ExitCommand exitCmd(area);
    area.add(new Button(353, 340, "flatokbtn.bmp", &exitCmd, false));
    area.add(new KeyAccel(SDLK_ESCAPE, &exitCmd));
    area.add(new KeyAccel(SDLK_RETURN, &exitCmd));
    area.run();
    storage->set("lastName", name);
    return name;
}


